﻿Public Class ItemData
    ' class for populating the list box
    ' absolute path of source file
    Private _source As String = Nothing
    ' name to be used, passed by application
    Private _name As String = Nothing
    Private _xtn As String = Nothing
    Private _tool As String = Nothing
    Private _args As String = Nothing
    Private _help As String = Nothing
    Public Sub New(ByVal source As String, ByVal name As String)
        _source = source
        _xtn = System.IO.Path.GetExtension(source)
        If _xtn = ".ini" Then
            _name = System.IO.Path.GetFileNameWithoutExtension(name)
        Else
            _name = name
            _tool = source
        End If
    End Sub
    Public Overrides Function ToString() As String
        Return _name
    End Function
    Public ReadOnly Property Name() As String
        Get
            Return _name
        End Get
    End Property
    Public ReadOnly Property Xtn() As String
        Get
            Return _xtn
        End Get
    End Property
    Public ReadOnly Property Tool() As String
        ' loaded on demand
        Get
            If _tool Is Nothing Then ProcessTool(_source)
            Return _tool
        End Get
    End Property
    Public ReadOnly Property Args() As String
        ' loaded on demand
        Get
            If _args Is Nothing Then ProcessTool(_source)
            Return _args
        End Get
    End Property
    Public ReadOnly Property Help() As String
        ' loaded on demand
        Get
            If _help Is Nothing Then ProcessHelp(_source)
            Return _help
        End Get
    End Property
    Private Sub ProcessTool(ByVal file As String)
        Const TB As String = "[Toolbox]"
        Const TE As String = "["
        Const P As String = "tool="
        Const A As String = "args="
        Const F As String = "file="
        Const Cmt As String = ";"

        Dim inToolbox As Boolean = True
        Dim ini() As String = Nothing

        With My.Computer.FileSystem
            If .FileExists(file) Then
                ini = System.IO.File.ReadAllLines(file)
            Else
                Exit Sub
            End If
        End With
        _tool = ""
        _args = ""
        For Each s In ini
            s = s.Trim()
            If InStr(s, Cmt) = 1 Then
                ' ignore this line - it is comment
            ElseIf InStr(s, TB) = 1 Then
                inToolbox = True
            ElseIf InStr(s, TE) = 1 Then
                inToolbox = False
            ElseIf InStr(s, P) = 1 Then
                If inToolbox Then
                    _tool = s.Substring(Len(P))
                End If
            ElseIf InStr(s, A) = 1 Then
                If inToolbox Then
                    _args &= s.Substring(Len(A)) & vbNewLine
                End If
            ElseIf InStr(s, F) = 1 Then
                If inToolbox Then
                    ' enclose in quotes if embedded spaces
                    _args &= Quoted(s.Substring(Len(F))) & vbNewLine
                End If
            End If
        Next
    End Sub
    Private Sub ProcessHelp(ByVal f As String)
        Const TB As String = "[Help]"
        Const TE As String = vbNewLine & "["

        Dim ini As String = ""

        With My.Computer.FileSystem
            If .FileExists(f) Then
                ini = .ReadAllText(f)
            End If
        End With
        ' find start of help section
        Dim i As Integer = InStr(ini, TB)
        If i = 0 Then
            ' no help
            _help = ""
            Exit Sub
        Else
            i += Len(TB) + 1 ' skip label
        End If

        ' find end of help section
        Dim l As Integer = InStr(i, ini, TE)
        ' end of help section might be end of file
        If l = 0 Then l = Len(ini)

        _help = ini.Substring(i, l - i) & vbNewLine
    End Sub
End Class
