﻿Public Class RichState
    Private _source As RichTextBox = Nothing
    Private _start As Integer = 0
    Private _length As Integer = 0
    Private _delstart As Integer = 0
    Private _dellength As Integer = 0
    Private _textLength As Integer = 0
    Public Sub New(ByRef rtx As RichTextBox)
        With rtx
            _source = rtx
            _start = .SelectionStart
            _length = .SelectionLength
            _delstart = .SelectionStart
            _dellength = .SelectionLength
            _textLength = .TextLength
        End With
    End Sub
    Public Sub DeleteText()
        If _dellength > 0 Then
            With _source
                .SelectionStart = _delstart
                .SelectionLength = _dellength
                .SelectedText = ""
                _start = .SelectionStart
                _length = .SelectionLength
                _textLength = .TextLength
            End With
        End If
    End Sub
    Public Sub InplaceMove(ByVal where As Integer)
        frmTB.MH.Report = "Inplace at: " & CStr(where)
        If where >= _start And where <= _start + _length Then
            ' move doesn't make sense
            _dellength = 0
            Return
        End If
        With _source
            Dim first As String = ""
            Dim text As String = .Text.Substring(_start, _length)
            If where > _start Then
                ' insert text first, delete point wil be stable
                frmTB.MH.Report = vbLf & "later " & .Text
                first = .Text.Substring(0, where)
                Dim last As String = .Text.Substring(where)
                frmTB.MH.Report = vbLf & "(1) " & first
                frmTB.MH.Report = vbLf & "(2) " & text
                frmTB.MH.Report = vbLf & "(3) " & last & vbLf
                .Text = first + text + last
                ' eliminate the dragged text
                .Text = .Text.Substring(0, _start) & .Text.Substring(_start + _length)
            End If
            If where < _start Then
                ' delete text first, insertion point will be stable
                Dim remainder As String = .Text.Substring(0, _start) & _
                        .Text.Substring(_start + _length)
                frmTB.MH.Report = vbLf & "earlier " & remainder
                first = remainder.Substring(0, where)
                Dim last As String = remainder.Substring(where)
                frmTB.MH.Report = vbLf & "(1) " & first
                frmTB.MH.Report = vbLf & "(2) " & text
                frmTB.MH.Report = vbLf & "(3) " & last & vbLf
                .Text = first + text + last
            End If
            _source.Select(first.Length, text.Length)
            Restore()
            _dellength = 0
        End With
    End Sub
    Public Sub Restore(Optional ByRef rtx As RichTextBox = Nothing)
        If rtx Is Nothing Then rtx = _source
        With rtx
            .SelectionStart = _start
            .SelectionLength = _length
        End With
    End Sub
    Public Sub Store()
        With _source
            _start = .SelectionStart
            _length = .SelectionLength
            _textLength = .TextLength
        End With
    End Sub
    Public ReadOnly Property Source() As RichTextBox
        Get
            Return _source
        End Get
    End Property
    Public ReadOnly Property SelectionStart() As Integer
        Get
            Return _start
        End Get
    End Property
    Public ReadOnly Property SelectionLength() As Integer
        Get
            Return (_length)
        End Get
    End Property
End Class
Public Class Mouse ' mostly for drag and drop
    ' this class is designed to implement drag and drop operations
    ' as such it has to handle most types of mouse events
    ' Mouse: Enter, move, up, down and leave
    ' Drag : Enter, over, drop and leave (no down equivalent)
    ' other keyboard and window events: keyup and gotfocus
    ' keydown and keypress are irrelevant
    ' I have tried to mimic Microsoft Word and Wordpad, particularly
    ' retaining selected text when regaining focus in a window
    ' and setting the cursor according to its position over text,
    ' selected text or the selection margin of a rich text box
    ' I use a hand cursor a indicate position over the margin
    Private _dropMode As Integer = 0        ' set during drag enter to 0, 1, or 2
    Private _currentDropMode As Integer = 0 ' set during drag over
    Private Const _Margin As Integer = 10   ' selection margin width in rich text box
    Private _dragging As RichState = Nothing    ' remembers text box settings
    Private _items As String = ""           ' use for listboxes
    ' flags for sequences
    Private _dragStart As Integer = 0       ' control mouse movement to trigger drag
    Private Const _Threshold As Integer = 2 ' control mouse movement to trigger drag
    Private _gettingfocus As RichState = Nothing
    ' debugging symbols 
    Private Const _Move As String = "." ' mouse
    Private Const _Over As String = "!" ' mouse
    Private Const _KeyDown As String = "D"  ' keyboard
    Private Const _KeyPress As String = "P" ' keyboard
    Private Const _KeyUp As String = "U"    ' keyboard
    ' text box for debug output, enabled by setting _report, calling ReportTo
    Private _report As RichTextBox = Nothing
    Public WriteOnly Property ReportTo() As RichTextBox
        Set(ByVal a As RichTextBox)
            _report = a     ' display debug reports here
        End Set
    End Property
    Public WriteOnly Property Report() As String
        Set(ByVal a As String)
            Const LIMIT As Integer = 4
            Static _count As Integer = 0
            If _report Is Nothing Then Exit Property
            If Not (a = _Move Or a = _Over Or a = _KeyDown Or a = _KeyPress) Then
                _count = 0      ' enable the display of mouse moves, etc
            Else
                If _count >= LIMIT Then Exit Property
                If a <> _KeyPress Then _count += 1 ' display another move
            End If
            _report.AppendText(a)
        End Set
    End Property
    Public ReadOnly Property GettingFocus() As Boolean
        Get
            Return Not (_gettingfocus Is Nothing)
        End Get
    End Property
    Public Sub DragEnter(ByRef a As String, ByRef e As System.Windows.Forms.DragEventArgs)
        Me.Report = vbLf & a & " DragEnter, "
        ' set up drag modes, they will remain the same in this control
        If _dragging Is Nothing Then
            _dropMode = DragDropEffects.Copy
        ElseIf e.Data.GetDataPresent(DataFormats.FileDrop) Then
            _dropMode = DragDropEffects.Copy
        ElseIf e.Data.GetDataPresent(DataFormats.Text) Then
            _dropMode = DragDropEffects.Move
        Else
            Me.Report = " None, "
            _dropMode = DragDropEffects.None
        End If
    End Sub
    Public Sub DragLeave()
        Me.Report = "DragLeave"
    End Sub
    Public Function DragOver()
        Me.Report = _Over
        ' _dropMode is set on drag enter, now check is control key is down
        _currentDropMode = _dropMode
        If _dropMode = DragDropEffects.Move Then
            If (Control.ModifierKeys And Keys.Control) = Keys.Control Then
                _currentDropMode = DragDropEffects.Copy
            Else
                _currentDropMode = DragDropEffects.Move
            End If
        End If
        Return _currentDropMode
    End Function
    Public Sub DragDrop(ByRef rtx As RichTextBox, ByRef e As System.Windows.Forms.DragEventArgs)
        Me.Report = vbLf & "Drop, "
        If _currentDropMode = DragDropEffects.Move _
            AndAlso Not _dragging Is Nothing _
            AndAlso ReferenceEquals(rtx, _dragging.Source) Then
            ' dragging and dropping within the same textbox
            ' special conditions apply
            _dragging.InplaceMove(rtx.SelectionStart)
        Else
            With rtx
                Dim toInsert As String = FetchData(e)
                ' Get start position to drop the text.
                Dim firstPart As String = .Text.Substring(0, .SelectionStart)
                ' get what remains after any selection
                Dim lastPart As String = .Text.Substring(firstPart.Length + .SelectionLength)
                .Text = firstPart & toInsert & lastPart
                .Select(firstPart.Length, toInsert.Length)
                Me.Report = If(_currentDropMode = DragDropEffects.Copy, " Copy", " Move")
            End With
        End If
    End Sub
    Private Function FetchData(ByRef e As System.Windows.Forms.DragEventArgs) As String
        Dim value As String = ""
        If e.Data.GetDataPresent(DataFormats.FileDrop) Then
            ' Assign the files to an array.
            Dim file() As String = e.Data.GetData(DataFormats.FileDrop)
            ' Loop through the array to build the new text
            For Each i In file
                value &= i & vbNewLine
            Next
        ElseIf e.Data.GetDataPresent(DataFormats.Text) Then
            value = e.Data.GetData(DataFormats.Text)
        End If
        Return value
    End Function
    Public Sub KeyDown(ByVal rtx As RichTextBox, ByRef e As System.Windows.Forms.KeyEventArgs)
        Report = _KeyDown
    End Sub
    Public Sub KeyPress(ByVal rtx As RichTextBox, ByRef e As System.Windows.Forms.KeyPressEventArgs)
        Report = _KeyPress
    End Sub
    Public Sub KeyUp(ByVal rtx As RichTextBox, ByRef e As System.Windows.Forms.KeyEventArgs)
        Report = _KeyUp
        If Not (_gettingfocus Is Nothing) Then
            _gettingfocus = Nothing
        End If
    End Sub
    Public Sub MouseEnter(ByRef rtx As RichTextBox, ByRef a As String)
        Me.Report = vbLf & a & " MouseEnter, "
    End Sub
    Public Sub MouseLeave()
        Me.Report = "MouseLeave"
    End Sub
    Public Sub MouseMove(ByRef rtx As RichTextBox, ByRef e As System.Windows.Forms.MouseEventArgs)
        Me.Report = _Move
        With rtx
            If _dragStart And .Cursor = Cursors.Arrow Then
                If _dragStart < _Threshold Then
                    ' move a bit more to start a drag
                    _dragStart += 1
                Else
                    ' now start the drag
                    _dragging = New RichState(rtx)
                    _dragStart = 0      ' clear start flag
                    ' Initiate dragging.
                    Dim r As DragDropEffects = _
                        .DoDragDrop(.SelectedText, DragDropEffects.All)
                    ''M3 29/10/2009
                    ' comes back here when the dragdrop has been completed
                    ' inside or outside this application
                    ' or if Esc pressed, or dropped on non-receptive control (i.e. none)
                    If r = DragDropEffects.Move Then
                        _dragging.DeleteText()
                    End If
                    Select Case r
                        Case DragDropEffects.Copy
                            Me.Report = "DragDrop terminated copy"
                        Case DragDropEffects.Move
                            Me.Report = "DragDrop terminated move"
                        Case DragDropEffects.None
                            Me.Report = "DragDrop terminated none"
                    End Select
                    rtx.SelectionStart = _dragging.SelectionStart
                    rtx.SelectionLength = _dragging.SelectionLength
                    _dragging = Nothing
                End If
            Else
                Dim margin As Integer = 0
                If .ShowSelectionMargin Then margin = _Margin
                If .SelectionLength > 0 Then
                    Dim c As Integer = .GetCharIndexFromPosition(e.Location)
                    If e.Location.X < margin Then
                        .Cursor = Cursors.Hand
                    ElseIf c >= .SelectionStart _
                    And c <= .SelectionStart + .SelectionLength - 1 _
                    And .Text.Substring(c, 1) <> vbLf Then
                        .Cursor = Cursors.Arrow
                    Else
                        .Cursor = Cursors.IBeam
                    End If
                ElseIf e.Location.X < margin Then
                    .Cursor = Cursors.Hand
                Else
                    .Cursor = Cursors.IBeam
                End If
            End If
        End With
    End Sub
    Public Sub MouseDown(ByRef rtx As RichTextBox, ByRef e As System.Windows.Forms.MouseEventArgs)
        Static L As String = "Ldown, "
        Static R As String = "Rdown, "
        Select Case e.Button
            Case MouseButtons.Left
                Me.Report = L
            Case MouseButtons.Right
                Me.Report = R
        End Select
        With rtx
            If Not (_gettingfocus Is Nothing) Then
                _gettingfocus.Restore(rtx)
                _gettingfocus = Nothing
            ElseIf .Cursor = Cursors.Arrow And e.Button = MouseButtons.Left Then
                _dragStart = 1
            End If
        End With
    End Sub
    Public Sub MouseUp()
        Me.Report = "Up, "
        ' a MouseDown sets _dragStart = 1 
        ' and allows a MouseMove to trigger a drag event
        ' a MouseUp before a drag is triggered will kill it
        _dragStart = 0
    End Sub
    Public Sub GotFocus(ByRef rtx As RichTextBox, Optional ByVal name As String = Nothing)
        Dim s As String = ""
        If rtx.TextLength > 0 Then
            s = "+text"
            _gettingfocus = New RichState(rtx)
            _gettingfocus.Store()
        End If
        If name Is Nothing Then name = rtx.Name
        Me.Report = vbLf & name & s & " Focus, "
    End Sub
    Private Sub ClearSource(ByRef src As ListBox)
        With src
            Dim n As Integer = 0
            Dim i As Integer = 0
            Dim l As Integer = 0
            While l > -1
                l = _items.IndexOf(vbLf, i)
                .Items.RemoveAt(CInt(_items.Substring(i, l - i)) - n)
                i = l + 1
                n += 1
            End While
            If i < _items.Length Then
                .Items.RemoveAt(CInt(_items.Substring(i)) - n)
            End If
        End With
    End Sub
End Class
