﻿Imports System.IO
Module ToolboxMain
    Public Const quote As String = """"       ' a single quote character
    Function ReadFile(ByVal path) As String
        Dim f = ""
        If path <> "" Then
            f = My.Computer.FileSystem.ReadAllText(path)
        End If
        Return f
    End Function
    Function GetFolder(ByRef path As String) As String
        Dim cd = New FolderBrowserDialog
        With cd
            .SelectedPath = path
            .ShowNewFolderButton = True
            If .ShowDialog() = DialogResult.Cancel Then Return ""
            Return .SelectedPath
        End With
    End Function
    Function GetOpenPath(ByVal path As String, ByVal restore As Boolean) As String
        Dim cd = New OpenFileDialog
        With cd
            .Filter = "All files (*.*)|*.*|TextFiles (*.txt)|*.txt"
            .FilterIndex = 2
            .InitialDirectory = path
            .Title = "Toolbox"
            .RestoreDirectory = restore
            If .ShowDialog() = DialogResult.Cancel Then
                Return ""
            End If
            Return .FileName
        End With
    End Function
    Function GetSavePath(ByVal path As String, ByVal restore As Boolean) As String
        Dim cd = New SaveFileDialog
        With cd
            .Filter = "Rich text files (*.rtf)|*.rtf|Plain text files (*.txt)|*.txt"
            .FilterIndex = 2
            .InitialDirectory = path
            .FileName = path
            .Title = "Toolbox"
            .RestoreDirectory = restore
            If .ShowDialog() = DialogResult.Cancel Then
                Return ""
            End If
            Return .FileName
        End With
    End Function
    Function GetToolsPath(ByVal full As Boolean) As String
        Const tools As String = "Tools\"
        Dim folder As String = Directory.GetCurrentDirectory() & tools
        If Not Directory.Exists(folder) Then
            Directory.CreateDirectory(folder)
        End If
        Return If(full, folder, tools)
    End Function
    Function GetTmpPath(ByVal full As Boolean) As String
        If full Then
            Return System.IO.Directory.GetCurrentDirectory() & "\Tmp\"
        Else : Return ".\Tmp\"
        End If
    End Function
    Function GetToolHelpPath(ByVal full As Boolean) As String
        If full Then
            Return System.IO.Directory.GetCurrentDirectory() & "\ToolHelp\"
        Else : Return ".\ToolHelp\"
        End If
    End Function
    Function GetHelpPath(ByVal full As Boolean) As String
        If full Then
            Return System.IO.Directory.GetCurrentDirectory() & "\ToolboxHelp\"
        Else : Return ".\ToolboxHelp\"
        End If
    End Function
    Function Quoted(ByRef s As String, Optional ByVal force As Boolean = False) As String
        If force Or InStr(s, " ") Then
            If Not (s.Substring(0, 1) = quote And s.Substring(Len(s) - 1, 1) = quote) Then
                s = quote & s & quote
            End If
        End If
        Return s
    End Function
    Function GetCommandLineArgs(ByRef s) As Boolean
        For Each i In My.Application.CommandLineArgs
            If i = "-d" Then
                frmTB.btnDebug.Visible = True
            Else
                s &= i & vbLf
            End If
        Next
        Return True
    End Function
    Function FindFiles(ByVal folder As String) As Collection
        ' return all the files in a folder as a collection
        With My.Computer.FileSystem
            If Not .DirectoryExists(folder) Then
                .CreateDirectory(folder)
            End If
            Dim t = .GetFiles(folder)
            Dim c = New Collection, s As String
            For Each s In t
                c.Add(.GetName(s))
            Next
            Return c
        End With
    End Function
    Private Function Redirection(ByVal s As String, ByVal r As String) As String
        If Left(s, 2) = ">>" Then Return Mid(s, 3)
        If Left(s, 1) = ">" Then Return r & s
        If Left(s, 1) = "<" Then Return Mid(s, 2)
        Return r & s
    End Function
    Sub AddRedirection(ByRef txt As RichTextBox, ByVal r As String)
        With txt
            Select Case .Lines.Count
                Case 0
                    .Text = r
                Case 1
                    .Text = Redirection(.Text, r)
                Case Else
                    Dim start = .SelectionStart
                    Dim s = Redirection(.SelectedText, r)
                    .SelectedText = s
                    .SelectionStart = start
                    .SelectionLength = Len(s)
            End Select
        End With
    End Sub
    Public Function FindControlByName(ByRef frm As Form, ByVal name As String) As Control
        Dim c As Control
        For Each c In frm.Controls
            If c.Name = name Then Return c
        Next
        Return Nothing
    End Function
    Public Function GetBrowser(ByRef lst As ListBox) As String
        Dim browser As String, q As Integer
        'q = lst.FindString("browser")
        'If q >= 0 Then Return lst.Items(q).Tool
        With My.Computer.Registry
            Dim key As String = "HKEY_CLASSES_ROOT\http\shell\open\command"
            browser = .GetValue(key, Nothing, Nothing)
            If browser Is Nothing Then
                Return ""
            End If
        End With
        q = InStr(browser, quote)
        If q > 0 Then browser = browser.Substring(q)
        q = InStr(browser, quote)
        If q > 0 Then browser = browser.Substring(0, q - 1)
        Return browser
    End Function
    Public Function FindText(ByRef l As String(), ByVal m As String) As Integer
        Dim t As Integer
        For t = 0 To l.GetUpperBound(0)
            If l(t) = m Then Return t
        Next
        Return -1
    End Function
End Module
